# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from design.plone.policy.testing import DESIGN_PLONE_POLICY_INTEGRATION_TESTING
from design.plone.policy.testing import (
    DESIGN_PLONE_POLICY_LIMIT_ROOT_ADDABLES_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import ISearchSchema
from Products.CMFPlone.interfaces import ISiteSchema
from Products.CMFPlone.interfaces.controlpanel import INavigationSchema
from zope.component import getUtility

import unittest


try:
    from Products.CMFPlone.utils import get_installer
except ImportError:
    get_installer = None


class TestSetup(unittest.TestCase):
    """Test that design.plone.policy is properly installed."""

    layer = DESIGN_PLONE_POLICY_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")

    def test_browserlayer(self):
        """Test that IDesignPlonePolicyLayer is registered."""
        from design.plone.policy.interfaces import IDesignPlonePolicyLayer
        from plone.browserlayer import utils

        self.assertIn(IDesignPlonePolicyLayer, utils.registered_layers())

    def test_show_excluded_items_disabled(self):
        registry = getUtility(IRegistry)
        navigation_settings = registry.forInterface(
            INavigationSchema, prefix="plone", check=False
        )
        self.assertFalse(navigation_settings.show_excluded_items)

    def test_sitemap_enabled(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        self.assertTrue(settings.enable_sitemap)

    def test_searchable_types(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISearchSchema, prefix="plone")
        self.assertEqual(
            sorted(settings.types_not_searched),
            sorted(
                (
                    "File",
                    "Image",
                    "Incarico",
                    "Modulo",
                    "Documento Personale",
                    "Bando Folder Deepening",
                    "Pratica",
                    "TempFolder",
                    "Collection",
                    "RicevutaPagamento",
                    "Link",
                    "Messaggio",
                    "Plone Site",
                    "Folder",
                    "Dataset",
                    "Discussion Item",
                    "Subsite",
                )
            ),
        )


class TestUninstall(unittest.TestCase):
    layer = DESIGN_PLONE_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")
        roles_before = api.user.get_roles(TEST_USER_ID)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer.uninstall_product("design.plone.policy")
        setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_product_uninstalled(self):
        """Test if design.plone.policy is cleanly uninstalled."""
        self.assertFalse(self.installer.is_product_installed("design.plone.policy"))

    def test_browserlayer_removed(self):
        """Test that IDesignPlonePolicyLayer is removed."""
        from design.plone.policy.interfaces import IDesignPlonePolicyLayer
        from plone.browserlayer import utils

        self.assertNotIn(IDesignPlonePolicyLayer, utils.registered_layers())


class TestSetupLimitRootAddables(unittest.TestCase):
    layer = DESIGN_PLONE_POLICY_LIMIT_ROOT_ADDABLES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")
        self.installer.install_product("design.plone.policy.limit_root_addables")

    def test_setup(self):
        plonesite = api.portal.get_tool("portal_types")["Plone Site"]
        self.assertTrue(plonesite.filter_content_types)
        self.assertEqual(
            sorted(plonesite.allowed_content_types),
            sorted(("Folder", "File", "Document", "Image", "Subsite")),
        )


class TestUninstallLimitRootAddables(unittest.TestCase):
    layer = DESIGN_PLONE_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        if get_installer:
            self.installer = get_installer(self.portal, self.layer["request"])
        else:
            self.installer = api.portal.get_tool("portal_quickinstaller")
        # roles_before = api.user.get_roles(TEST_USER_ID)
        # setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer.install_product("design.plone.policy.limit_root_addables")
        # setRoles(self.portal, TEST_USER_ID, roles_before)

    def test_uninstall(self):
        plonesite = api.portal.get_tool("portal_types")["Plone Site"]
        self.installer.uninstall_product("design.plone.policy.limit_root_addables")
        self.assertFalse(plonesite.filter_content_types)
        self.assertEqual(
            plonesite.allowed_content_types,
            (),
        )



# -*- coding: utf-8 -*-
from AccessControl.unauthorized import Unauthorized
from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
from plone import api
from plone.registry.interfaces import IRegistry
from plone.restapi.interfaces import ISerializeToJsonSummary
from plone.restapi.services import Service
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import ISearchSchema
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse

import json


@implementer(IPublishTraverse)
class SearchFiltersGet(Service):
    def __init__(self, context, request):
        super(SearchFiltersGet, self).__init__(context, request)

    def get_portal_types(self):
        ttool = api.portal.get_tool("portal_types")
        ptool = api.portal.get_tool("plone_utils")
        registry = getUtility(IRegistry)
        search_settings = registry.forInterface(ISearchSchema, prefix="plone")
        types_not_searched = search_settings.types_not_searched
        types = [
            {
                "label": translate(ttool[t].Title(), context=self.request),
                "id": t,
            }
            for t in ptool.getUserFriendlyTypes()
            if t not in types_not_searched
        ]
        return sorted(types, key=lambda k: k["label"])

    def reply(self):
        settings = api.portal.get_registry_record(
            "search_sections", interface=IDesignPloneSettings, default="[]"
        )
        utils = getToolByName(self.context, "plone_utils")

        sections = []
        for setting in json.loads(settings or "[]"):
            items = []
            for section_settings in setting.get("items") or []:
                for uid in section_settings.get("linkUrl") or []:
                    try:
                        section = api.content.get(UID=uid)
                    except Unauthorized:
                        # private folder
                        continue
                    if not section:
                        continue
                    item_infos = getMultiAdapter(
                        (section, self.request),
                        ISerializeToJsonSummary,
                    )()
                    if section_settings.get("expandItems", True):
                        children = section.listFolderContents(
                            contentFilter={"portal_type": utils.getUserFriendlyTypes()}
                        )
                        item_infos["items"] = [
                            getMultiAdapter(
                                (x, self.request),
                                ISerializeToJsonSummary,
                            )()
                            for x in children
                        ]
                    else:
                        # do not expand childrens, the only item is the section/container itself
                        item_infos["items"] = [
                            getMultiAdapter(
                                (section, self.request),
                                ISerializeToJsonSummary,
                            )()
                        ]
                    item_infos["title"] = section_settings.get("title", "")
                    items.append(item_infos)
            if items:
                sections.append(
                    {
                        "rootPath": setting.get("rootPath", ""),
                        "items": items,
                    }
                )
        topics = [
            getMultiAdapter(
                (brain, self.request),
                ISerializeToJsonSummary,
            )()
            for brain in api.content.find(
                portal_type="Pagina Argomento",
                sort_on="sortable_title",
                sort_order="ascending",
            )
        ]
        return {
            "sections": sections,
            "topics": topics,
            "portal_types": self.get_portal_types(),
        }



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from collective.volto.blocksfield.field import BlocksField
from copy import deepcopy
from design.plone.policy.interfaces import IDesignPlonePolicySettings
from design.plone.policy.setuphandlers import disable_searchable_types
from design.plone.policy.setuphandlers import set_default_subsite_colors
from design.plone.policy.utils import create_default_blocks
from plone import api
from plone.app.upgrade.utils import installOrReinstallProduct
from plone.dexterity.utils import iterSchemata
from plone.registry.interfaces import IRegistry
from plone.restapi.behaviors import IBlocks
from Products.CMFPlone.interfaces import IFilterSchema
from Products.CMFPlone.interfaces import ISelectableConstrainTypes
from zope.component import getUtility
from zope.schema import getFields

import json
import logging


logger = logging.getLogger(__name__)

DEFAULT_PROFILE = "profile-design.plone.policy:default"


def update_profile(context, profile, run_dependencies=True):
    context.runImportStepFromProfile(DEFAULT_PROFILE, profile, run_dependencies)


def update_types(context):
    update_profile(context, "typeinfo")


def update_rolemap(context):
    update_profile(context, "rolemap")


def update_registry(context):
    update_profile(context, "plone.app.registry", run_dependencies=False)


def update_catalog(context):
    update_profile(context, "catalog")


def update_controlpanel(context):
    update_profile(context, "controlpanel")


def to_1200(context):
    def fix_field_name(blocks):
        """ """
        found = False
        for block in blocks.values():
            if block.get("@type", "") == "form" and block.get("to", ""):
                block["default_to"] = block.get("to", "")
                del block["to"]
                found = True
        return found

    installOrReinstallProduct(api.portal.get(), "collective.volto.formsupport")
    logger.info("Changing form block fields.")
    i = 0
    brains = api.content.find(object_provides="plone.restapi.behaviors.IBlocks")
    tot = len(brains)
    fixed_items = []
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item = brain.getObject()
        blocks = deepcopy(getattr(item, "blocks", {}))
        if blocks:
            to_update = fix_field_name(blocks)
            if to_update:
                item.blocks = blocks
                fixed_items.append(brain.getPath())

    logger.info("Finish")
    if fixed_items:
        logger.info("Updated items:")
        for fixed in fixed_items:
            logger.info("- {}".format(fixed))
    else:
        logger.info("No items affected.")


def to_1300(context):
    disable_searchable_types()


def to_1400(context):
    old = api.portal.get_registry_record(name="design.plone.policy.twitter_token")
    context.runAllImportStepsFromProfile("profile-design.plone.policy:to_1400")
    update_registry(context)

    if old:
        api.portal.set_registry_record(
            "twitter_token", old, interface=IDesignPlonePolicySettings
        )


def to_1500(context):
    """This upgrade handles that a type "Bando" is now  addmitted by default
    inside the folder "Documenti e Dati".
    This method just ADD THE CONTENT TYPE to the current list of types that you
    can add inside that folder only if it's not already there.
    tp#17807
    """

    doc_brains = api.content.find(portal_type="Document")
    doc_e_dati_list = [x for x in doc_brains if x.Title == "Documenti e dati"]
    if len(doc_e_dati_list) == 1:
        doc_e_dati = doc_e_dati_list[0].getObject()

        constraints = ISelectableConstrainTypes(doc_e_dati)
        allowed_types = constraints.getLocallyAllowedTypes()
        if "Bando" not in allowed_types:
            allowed_types.append("Bando")
            constraints.setLocallyAllowedTypes(allowed_types)

            logger.info("Enabled 'Bando' inside 'Ducumenti e dati' folder.")
        else:
            logger.info(
                "'Bando' already enabled in 'Ducumenti e dati' folder,"
                " not changes needed."
            )
    else:
        logger.warning(
            "More than one Document with title 'Documenti e dati'. "
            "Type 'Bando' inside 'Ducumenti e dati' folder not "
            "enabled."
        )


def to_1600(context):
    installOrReinstallProduct(api.portal.get(), "collective.volto.subsites")
    set_default_subsite_colors()
    logger.info("### CHANGE SUBSITE COLOR light-blue => teal")
    subsites = api.content.find(portal_type="Subsite")
    for brain in subsites:
        subsite = brain.getObject()
        if getattr(subsite, "subsite_css_class", "") == "light-blue":
            subsite.subsite_css_class = "teal"
            logger.info("- {}".format(brain.getURL()))


def to_1700(context):
    installOrReinstallProduct(api.portal.get(), "collective.volto.subfooter")


def to_1800(context):
    installOrReinstallProduct(api.portal.get(), "rer.customersatisfaction")


def to_1900(context):
    installOrReinstallProduct(api.portal.get(), "redturtle.faq")


def to_1910(context):
    allowed_sizes = api.portal.get_registry_record("plone.allowed_sizes")
    new_sizes = []
    skip = ["listing 16:16", "icon 32:32", "tile 64:64"]
    for size in allowed_sizes:
        if size in skip:
            new_sizes.append(size)
        else:
            name_height, width = size.split(":")
            if width != "65536":
                new_sizes.append("{}:{}".format(name_height, "65536"))
            else:
                new_sizes.append(size)
    if "midi 300:65536" not in new_sizes:
        new_sizes.insert(new_sizes.index("mini 200:65536") + 1, "midi 300:65536")
    api.portal.set_registry_record("plone.allowed_sizes", new_sizes)


def to_2000(context):  # noqa: C901
    logger.info("### START CONVERSION FORMS: enable honeypot ###")

    def fix_block(blocks):
        found = False
        for block in blocks.values():
            if block.get("@type", "") == "form":
                captcha = block.get("captcha", "")
                if captcha != "honeypot":
                    block["captcha"] = "honeypot"
                    found = True
        return found

    forms = []

    # fix root
    portal = api.portal.get()
    try:
        portal_blocks = json.loads(portal.blocks)
        res = fix_block(portal_blocks)
        if res:
            forms.append("site root")
            portal.blocks = json.dumps(portal_blocks)
    except TypeError:
        # This happens when the site was already Plone 6
        # and the blocks field is already a dictionary.
        # Since it's Plone 6, the site root will be in the catalog query below,
        # so just skip this step.
        pass

    # fix blocks in contents
    pc = api.portal.get_tool(name="portal_catalog")
    brains = pc()
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info("Progress: {}/{}".format(i, tot))
        item = aq_base(brain.getObject())
        for schema in iterSchemata(item):
            for name, field in getFields(schema).items():
                if name == "blocks":
                    blocks = deepcopy(item.blocks)
                    if blocks:
                        res = fix_block(blocks)
                        if res:
                            forms.append(brain.getURL())
                            item.blocks = blocks
                elif isinstance(field, BlocksField):
                    value = deepcopy(field.get(item))
                    if not value:
                        continue
                    try:
                        blocks = value.get("blocks", {})
                    except AttributeError:
                        logger.warning(
                            "[RICHTEXT] - {} (not converted)".format(brain.getURL())
                        )
                    if blocks:
                        res = fix_block(blocks)
                        if res:
                            forms.append(brain.getURL())
                            setattr(item, name, value)

    logger.info(f"Found {len(forms)} forms.")
    for url in forms:
        logger.info(f"- {url}")


def to_2010(context):
    registry = getUtility(IRegistry)
    settings = registry.forInterface(IFilterSchema, prefix="plone")
    settings.custom_attributes = settings.custom_attributes + ["data-element"]


def update_folders(context, CHANGES=[], NEW_ITEMS=[]):
    GREEN = "\033[92m"
    ENDC = "\033[0m"
    RED = "\033[91m"

    portal = api.portal.get()
    portal_name = portal.getId()
    for item in CHANGES:
        try:
            folder = portal.restrictedTraverse("{}{}".format(portal_name, item))
            old_title = folder.title
            old_path = "/".join(folder.getPhysicalPath())
            api.content.rename(obj=folder, new_id=CHANGES[item][1])
            folder.title = CHANGES[item][0]
            # folder.reindexObject(idx=['id', 'title', 'getId', 'SearchableText'])
            # Who know the exact number of index we need to update?
            folder.reindexObject()
            logger.info(
                "{} Modificato {} ({}) in {} {}".format(
                    GREEN, old_title, old_path, folder.title, ENDC
                )
            )
        except KeyError:
            logger.info("{} Impossibile modificare {}{}".format(RED, item, ENDC))

    for item in NEW_ITEMS:
        folder = portal.restrictedTraverse("{}{}".format(portal_name, item[0]))
        if item[2] not in folder:
            new = api.content.create(
                type="Document", id=item[2], title=item[1], container=folder
            )
            logger.info(
                "{} Creato {} in {} {}".format(
                    GREEN, item[1], "/".join(new.getPhysicalPath()), ENDC
                )
            )
        else:
            logger.info(
                "{} {} esiste già in {} {}".format(
                    RED, item[1], "/".join(folder.getPhysicalPath()), ENDC
                )
            )


def update_folders_name_for_pnrr(context):
    CHANGES = {
        "/servizi/tributi-e-finanze": (
            "Tributi, finanze e contravvenzioni",
            "tributi-finanze-e-contravvenzioni",
        ),
        "/servizi/agricoltura": ("Agricoltura e pesca", "agricoltura-e-pesca"),
    }
    NEW_ITEMS = [
        ("/servizi", "Imprese e commercio", "imprese-e-commercio"),
    ]
    update_folders(context, CHANGES, NEW_ITEMS)


def to_3000(context):
    # run design.plone.contenttypes steps
    context.upgradeProfile(
        "profile-design.plone.contenttypes:default", dest="7001", quiet=False
    )
    update_folders_name_for_pnrr(context)


def to_3001(context):
    # run design.plone.contenttypes steps
    context.upgradeProfile(
        "profile-design.plone.contenttypes:default", dest="7008", quiet=False
    )
    context.runImportStepFromProfile(
        "profile-design.plone.contenttypes:default",
        "plone.app.registry",
        run_dependencies=False,
    )
    installOrReinstallProduct(api.portal.get(), "collective.feedback")


def to_3100(context):
    already_modified = []
    not_modified = []
    brains = api.content.find(object_provides=IBlocks.__identifier__)
    tot = len(brains)
    i = 0
    for brain in brains:
        i += 1
        if i % 1000 == 0:
            logger.info(f"Progress: {i}/{tot}")
        item = brain.getObject().aq_base
        blocks = getattr(item, "blocks", {})
        blocks_layout = getattr(item, "blocks_layout", {})

        if blocks and blocks_layout == {"items": []}:
            # case where document has been created without blocks, and has been modified
            item.blocks_layout = {"items": [x for x in blocks.keys()]}
            already_modified.append(brain.getPath())
            continue
        if not blocks and blocks_layout == {"items": []}:
            create_default_blocks(item)
            not_modified.append(brain.getPath())

    if already_modified:
        logger.info("### Items that were already modified ###")
        for i, path in enumerate(already_modified):
            logger.info(f"[{i+1}/{len(already_modified)}] - {path}")

    if not_modified:
        logger.info("### Items that were not modified ###")
        for i, path in enumerate(not_modified):
            logger.info(f"[{i+1}/{len(not_modified)}] - {path}")


def to_3101(context):
    for brain in api.portal.get_tool("portal_catalog")():
        item = aq_base(brain.getObject())

        for schema in iterSchemata(item):
            for name, field in getFields(schema).items():
                if name == "blocks":
                    logger.info(
                        f"[3100 - 3101] Deleting twitter blocks if "
                        f"exist from {'/'.join(item.getPhysicalPath())}"
                    )

                    twitter_block_uids = []
                    blocks = deepcopy(item.blocks)
                    blocks_layout = deepcopy(getattr(item, "blocks_layout", {}))

                    for key, block in deepcopy(blocks).items():
                        if block.get("@type", "") == "twitter_posts":
                            twitter_block_uids.append(key)
                            del blocks[key]

                    for block_uid in [*item.blocks_layout.get("items", [])]:
                        if block_uid in twitter_block_uids:
                            blocks_layout["items"].remove(block_uid)

                    item.blocks = blocks

                    if blocks_layout:
                        item.blocks_layout = blocks_layout

                elif isinstance(field, BlocksField):
                    value = deepcopy(field.get(item))
                    if not value:
                        continue
                    try:
                        blocks = value.get("blocks", {})

                    except AttributeError:
                        logger.warning(
                            "[BLOCK] - {} (not converted)".format(brain.getURL())
                        )

                    if blocks:
                        logger.info(
                            f"[3100 - 3101] Deleting twitter blocks if "
                            f"exist from {'/'.join(item.getPhysicalPath())}"
                        )

                        twitter_block_uids = []
                        blocks = deepcopy(getattr(item, "blocks", None))

                        blocks_layout = deepcopy(getattr(item, "blocks_layout", None))

                        if blocks and blocks_layout:
                            for key, block in deepcopy(blocks).items():
                                if block.get("@type", "") == "twitter_posts":
                                    twitter_block_uids.append(key)
                                    del blocks[key]

                            for block_uid in [*item.blocks_layout.get("items", [])]:
                                if block_uid in twitter_block_uids:
                                    blocks_layout["items"].remove(block_uid)

                            item.blocks = blocks

                            if blocks_layout:
                                item.blocks_layout = blocks_layout



# -*- coding: utf-8 -*-
from iw.rejectanonymous import rejectAnonymous


def insertRejectAnonymousHook(portal, event):
    """force authentication for request with X-ForceAuth header"""
    if event.request.getHeader("X-ForceAuth"):
        event.request.post_traverse(rejectAnonymous, (portal, event.request))



# -*- coding: utf-8 -*-
from design.plone.policy.testing import DESIGN_PLONE_POLICY_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest


class TestInitialStructureCreation(unittest.TestCase):
    layer = DESIGN_PLONE_POLICY_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_plone_base_interfaces_syndication_ISiteSyndicationSettings_show_author_info(  # noqa
        self,
    ):
        self.assertFalse(
            api.portal.get_registry_record(
                "plone.base.interfaces.syndication.ISiteSyndicationSettings.show_author_info"  # noqa
            )
        )



# -*- coding: utf-8 -*-
"""Init and utils."""
from .patches import apply as apply_patches
from .sensitive import apply
from zope.i18nmessageid import MessageFactory


_ = MessageFactory("design.plone.policy")
apply()
apply_patches()



# -*- coding: utf-8 -*-
"""
We use this file to change the base behavior of collective.volto.formsupport
to support some new feature:
    - limit on form submit
    - unique field in one form

Why do we use monkeypatch instead of overriding the classes?
Because it's temporary, until collective.volto.formsupport can support backend
validation for data
"""
from collective.volto.formsupport import _
from collective.volto.formsupport.datamanager.catalog import FormDataStore
from collective.volto.formsupport.interfaces import IFormDataStore
from collective.volto.formsupport.restapi.services.form_data.csv import (
    FormDataExportGet,
)
from collective.volto.formsupport.restapi.services.submit_form.post import logger
from collective.volto.formsupport.restapi.services.submit_form.post import (
    PostEventService,
)
from collective.volto.formsupport.restapi.services.submit_form.post import SubmitPost
from datetime import datetime
from io import StringIO
from plone.protect.interfaces import IDisableCSRFProtection
from plone.restapi.serializer.converters import json_compatible
from souper.soup import Record
from zExceptions import BadRequest
from zope.component import getMultiAdapter
from zope.event import notify
from zope.i18n import translate
from zope.interface import alsoProvides

import csv


SKIP_ATTRS = ["block_id", "fields_labels", "fields_order"]


def get_data(self):
    store = getMultiAdapter((self.context, self.request), IFormDataStore)
    sbuf = StringIO()
    fixed_columns = ["date"]
    columns = []
    # start patch
    custom_colums = []
    if self.form_block.get("limit", None) is not None:
        limit = int(self.form_block["limit"])
        if limit > -1:
            custom_colums.append("waiting_list")
    # end patch

    rows = []
    # start patch
    for index, item in enumerate(reversed(store.search())):
        # end patch
        data = {}
        fields_labels = item.attrs.get("fields_labels", {})
        for k in self.get_ordered_keys(item):
            if k in SKIP_ATTRS:
                continue
            value = item.attrs.get(k, None)
            label = fields_labels.get(k, k)
            if label not in columns and label not in fixed_columns:
                columns.append(label)
            data[label] = json_compatible(value)
        for k in fixed_columns:
            # add fixed columns values
            value = item.attrs.get(k, None)
            data[k] = json_compatible(value)

        # start patch
        if "waiting_list" in custom_colums:
            data.update(
                {
                    "waiting_list": (
                        translate(_("yes_label", default="Yes"))
                        if not (index < limit)
                        else translate(_("no_label", default="No"))
                    )
                }
            )
        # end patch

        rows.append(data)
    columns.extend(fixed_columns)
    columns.extend(custom_colums)
    writer = csv.DictWriter(sbuf, fieldnames=columns, quoting=csv.QUOTE_ALL)
    writer.writeheader()
    for row in rows:
        writer.writerow(row)
    res = sbuf.getvalue()
    sbuf.close()
    return res


def patch_FormDataExportGet_get_data():
    logger.info(
        "Patch get_data methos of class FormDataExporterGet from collective.volto.formsupport"  # noqa
    )
    FormDataExportGet.get_data = get_data


def reply(self):
    self.validate_form()

    # start patch
    self.store_action = self.block.get("store", False)
    self.send_action = self.block.get("send", [])
    self.submit_limit = int(self.block.get("limit", "-1"))

    # Disable CSRF protection
    alsoProvides(self.request, IDisableCSRFProtection)

    notify(PostEventService(self.context, self.form_data))
    data = self.form_data.get("data", [])
    # end patch

    if self.send_action:
        try:
            self.send_data()
        except BadRequest as e:
            raise e
        except Exception as e:
            logger.exception(e)
            message = translate(
                _(
                    "mail_send_exception",
                    default="Unable to send confirm email. Please retry later or contact site administrator.",  # noqa
                ),
                context=self.request,
            )
            self.request.response.setStatus(500)
            return dict(type="InternalServerError", message=message)
    # start patch
    if self.store_action:
        try:
            data = self.store_data()
        except ValueError as e:
            logger.exception(e)
            message = translate(
                _(
                    "save_data_exception",
                    default="Impossibile salvare i dati. I campi '${fields}' non sono univoci.",  # noqa
                    mapping={"fields": e.args[0]},
                ),
                context=self.request,
            )
            self.request.response.setStatus(500)
            return dict(type="InternalServerError", message=message)

    return {"data": data}
    # end patch


def store_data(self):
    store = getMultiAdapter((self.context, self.request), IFormDataStore)
    # start patch
    data = {"form_data": self.filter_parameters()}

    res = store.add(data=data)
    if not res:
        raise BadRequest("Unable to store data")

    waiting_list = (
        self.submit_limit is not None and -1 < self.submit_limit < self.count_data()
    )
    data.update({"waiting_list": waiting_list})

    return data


def count_data(self):
    store = getMultiAdapter((self.context, self.request), IFormDataStore)
    return store.count()


# end patch


def patch_SubmitPost_reply():
    logger.info(
        "Patch reply method of class SubmitPost from collective.volto.formsupport"
    )
    SubmitPost.reply = reply
    SubmitPost.store_data = store_data
    SubmitPost.count_data = count_data


def add(self, data):
    form_fields = self.get_form_fields()
    if not form_fields:
        logger.error(
            'Block with id {} and type "form" not found in context: {}.'.format(
                self.block_id, self.context.absolute_url()
            )
        )
        return None

    fields = {
        x["field_id"]: x.get("custom_field_id", x.get("label", x["field_id"]))
        for x in form_fields
    }
    record = Record()
    fields_labels = {}
    fields_order = []
    # start patch
    for field_data in data["form_data"]:
        # end patch
        field_id = field_data.get("field_id", "")
        value = field_data.get("value", "")
        if field_id in fields:
            record.attrs[field_id] = value
            fields_labels[field_id] = fields[field_id]
            fields_order.append(field_id)
    record.attrs["fields_labels"] = fields_labels
    record.attrs["fields_order"] = fields_order
    record.attrs["date"] = datetime.now()
    record.attrs["block_id"] = self.block_id

    # start patch
    keys = [(x["field_id"], x["label"]) for x in form_fields if x.get("unique", False)]
    if keys:
        saved_data = self.soup.data.values()
        for saved_record in saved_data:
            unique = False
            for key in keys:
                if record.attrs.storage[key[0]] != saved_record.attrs.storage.get(
                    key[0], None
                ):
                    unique = True
                    break

            if not unique:
                raise ValueError(f" {', '.join([x[1] for x in keys])}")
        # end patch

    return self.soup.add(record)


# start patch
def count(self, query=None):
    records = []
    if not query:
        records = self.soup.data.values()

    return len(records)


# end patch


def patch_FormDataStore_methods():
    logger.info(
        "Patch method add of FormDataStore class for collective.volto.formsupport"
    )
    FormDataStore.add = add
    logger.info(
        "Add method count of FormDataStore class for collective.volto.formsupport"
    )
    FormDataStore.count = count



from collective.volto.formsupport.testing import (  # noqa: E501,
    VOLTO_FORMSUPPORT_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from Products.MailHost.interfaces import IMailHost
from zope.component import getUtility

import transaction
import unittest


class TestLimitMailStore(unittest.TestCase):
    layer = VOLTO_FORMSUPPORT_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.mailhost = getUtility(IMailHost)

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.document = api.content.create(
            type="Document",
            title="Example context",
            container=self.portal,
        )

        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {"@type": "form"},
        }
        self.document_url = self.document.absolute_url()
        transaction.commit()

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

        # set default block
        self.document.blocks = {
            "text-id": {"@type": "text"},
            "form-id": {"@type": "form"},
        }
        transaction.commit()

    def submit_form(self, data):
        url = f"{self.document_url}/@submit-form"
        response = self.api_session.post(
            url,
            json=data,
        )
        transaction.commit()
        return response

    def test_limit_submit(self):
        self.document.blocks = {
            "form-id": {
                "@type": "form",
                "store": True,
                "limit": 1,
                "subblocks": [
                    {
                        "label": "Message",
                        "field_id": "message",
                        "field_type": "text",
                    },
                    {
                        "label": "Name",
                        "field_id": "name",
                        "field_type": "text",
                    },
                ],
            },
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "data": [
                    {"field_id": "message", "value": "just want to say hi"},
                    {"field_id": "name", "value": "John"},
                    {"field_id": "foo", "value": "skip this"},
                ],
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()
        self.assertEqual(response.status_code, 200)

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "data": [
                    {"field_id": "message", "value": "just want to say hi"},
                    {"field_id": "name", "value": "John"},
                    {"field_id": "foo", "value": "skip this"},
                ],
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()
        self.assertEqual(response.status_code, 200)
        self.assertTrue(response.json()["data"]["waiting_list"])

    def test_unique_field(self):
        self.document.blocks = {
            "form-id": {
                "@type": "form",
                "store": True,
                "subblocks": [
                    {
                        "label": "Message",
                        "field_id": "message",
                        "field_type": "text",
                    },
                    {
                        "label": "Name",
                        "field_id": "name",
                        "field_type": "text",
                        "unique": True,
                    },
                ],
            },
        }
        transaction.commit()

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "data": [
                    {"field_id": "message", "value": "just want to say hi"},
                    {"field_id": "name", "value": "John"},
                    {"field_id": "foo", "value": "skip this"},
                ],
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()
        self.assertEqual(response.status_code, 200)

        response = self.submit_form(
            data={
                "from": "john@doe.com",
                "data": [
                    {"field_id": "message", "value": "just want to say hi"},
                    {"field_id": "name", "value": "John"},
                    {"field_id": "foo", "value": "skip this"},
                ],
                "subject": "test subject",
                "block_id": "form-id",
            },
        )
        transaction.commit()
        self.assertEqual(response.status_code, 500)
        # test message is not fair because it's a translation, in another package
        message = response.json()["message"]
        self.assertTrue("Value not unique" in message or "non sono univoci" in message)



# -*- coding: utf-8 -*-
from design.plone.policy.testing import DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from transaction import commit
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid.interfaces import IIntIds

import unittest


class BandiSearchFiltersAPITest(unittest.TestCase):
    layer = DESIGN_PLONE_POLICY_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        intids = getUtility(IIntIds)
        self.uo_public_1 = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="UO 1"
        )
        self.uo_public_2 = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="UO 2"
        )
        self.uo_private = api.content.create(
            container=self.portal, type="UnitaOrganizzativa", title="UO 3"
        )
        self.bando_public_1 = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando 1",
            subject=("foo"),
            ufficio_responsabile=[RelationValue(intids.getId(self.uo_public_1))],
        )
        self.bando_public_2 = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando 2",
            subject=("foo", "bar"),
            ufficio_responsabile=[
                RelationValue(intids.getId(self.uo_public_2)),
                RelationValue(intids.getId(self.uo_private)),
            ],
        )
        self.bando_private = api.content.create(
            container=self.portal,
            type="Bando",
            title="Bando 3",
            subject=("foo", "baz"),
            ufficio_responsabile=[RelationValue(intids.getId(self.uo_public_1))],
        )

        api.content.transition(obj=self.uo_public_1, transition="publish")
        api.content.transition(obj=self.uo_public_2, transition="publish")
        api.content.transition(obj=self.bando_public_1, transition="publish")
        api.content.transition(obj=self.bando_public_2, transition="publish")

        commit()

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

    def test_endpoint_exists(self):
        response = self.api_session.get("/@bandi-search-filters")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers.get("Content-Type"), "application/json")

    def test_endpoint_return_list_of_offices_based_on_permissions(self):
        response = self.api_session.get("/@bandi-search-filters").json()

        self.assertIn("offices", response)
        offices = response["offices"]

        self.assertEqual(len(offices), 3)

        response = self.anon_api_session.get("/@bandi-search-filters").json()

        self.assertIn("offices", response)
        offices = response["offices"]
        self.assertEqual(len(offices), 2)

    def test_endpoint_return_list_of_subjects_based_on_permissions(self):
        response = self.api_session.get("/@bandi-search-filters").json()

        self.assertIn("subjects", response)
        subjects = [x["UID"] for x in response["subjects"]]
        self.assertEqual(len(subjects), 3)
        self.assertEqual(subjects, ["bar", "baz", "foo"])

        response = self.anon_api_session.get("/@bandi-search-filters").json()

        self.assertIn("subjects", response)
        subjects = [x["UID"] for x in response["subjects"]]
        self.assertEqual(len(subjects), 2)
        self.assertEqual(subjects, ["bar", "foo"])
